/*
 * Vinyl widget example:
 * Name: Range Slider
 * Developer: Vinyl
 * Date: 4/28/2021
 */
define(function () {

    var install = function (holderElement, context) {

        var sliderElement = holderElement.firstElementChild.firstElementChild;
        var row = context.getRow();

        // get the parameter from the row. Will use defaults of 0, 100, and 1.
        var min = parseInt(context.getParameter("Min"), 10);
        var max = parseInt(context.getParameter("Max"), 10);
        var step = parseFloat(context.getParameter("Step"));

        // Get the cells of the high and low slider values
        var lowColumn = context.getParameter("LowColumn");
        var lowCell = row.getCellByColumnName(lowColumn);

        var highColumn = context.getParameter("HighColumn");
        var highCell = row.getCellByColumnName(highColumn);

        // Figure out how many decimal places the step has - this will determine
        // the handle labels.
        var decimalsInLabel = Number.isInteger(step) ? 0 : 2;

        // install the slider library and the wNumb dependency
        var sliderLibrary = context.loadedDependencies[0];
        var wNumb = context.loadedDependencies[1];

        // Create the slider
        sliderLibrary.create(sliderElement, {
            start: [
                lowCell.value === null ? 0 : lowCell.value,
                highCell.value === null ? 0 : highCell.value
            ],
            range: {
                'min': min,
                'max': max
            },
            pips: {
                mode: 'positions',
                values: [0, 20, 40, 60, 80, 100],
                density: 3,
                stepped: true
            },
            step: step,
            tooltips: [
                wNumb({decimals: decimalsInLabel}),
                wNumb({decimals: decimalsInLabel})
            ],
            connect: true
        });

        // slider change event sets the Vinyl cell values
        sliderElement.noUiSlider.on('change', function (values, handle) {
            lowCell.value = values[0];
            highCell.value = values[1];
        });
    };

    var uninstall = function (holderElement, context) {
        var sliderElement = holderElement.childNodes[0];

        sliderElement.noUiSlider.off('change');

        sliderElement.noUiSlider.destroy();
    };

    return {
        callbacks: {
            events: {
                // adding and removing
                install: install,
                uninstall: uninstall
            }
        }
    };
});